package ch.epfl.lamp.minischeme;

import java.io._;
import scala.collection.immutable.{Map,ListMap};

object Main {
  def main(argsArray: Array[String]): Unit = {
    val args = List.fromArray(argsArray);
    val Pair(inputs, outputs) =
      args partition { fileName => fileName endsWith ".scm" };
    val iStreams =
      if (inputs.isEmpty) List(System.in)
      else inputs map { f => new FileInputStream(f) };
    val oStream =
      if (outputs.isEmpty) System.out
      else new PrintStream(new FileOutputStream(outputs.head));

    val parser = new Parser(new Scanner(iStreams));
    val program = parser.program;
    val primitives =
      List("$+","$-","$*","$/","$%",
           "$=","$<","$<=","$not",
           "$alloc","$set","$get",
           "$read-int","$print-int","$read-char","$print-char");
    val globalEnv: Map[String, Symbol] =
      ListMap.Empty[String,Symbol] incl
      (primitives map { p => Pair(p, new Symbol(p, PrimitiveKind)) });
    NameAnalyzer(globalEnv, program);
    Generator(program, oStream);
  }
}
