/*
 * MiniVM project (c) LAMP 2006
 * 
 * Interactive mode interface.
 * 
 * $Id: prompt.c,v 1.1 2006/03/16 15:55:26 dragos Exp $
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <readline/readline.h>
#include <readline/history.h>

#include "../config.h"
#include "vm.h"

void
print_regs();

void
print_help();

/* A static variable for holding the line. */
static char *line_read = (char *)NULL;

/* Read a string, and return a pointer to it.
   Returns NULL on EOF. */
char *
rl_gets ()
{
  /* If the buffer has already been allocated,
     return the memory to the free pool. */
  if (line_read)
    {
      free (line_read);
      line_read = (char *)NULL;
    }

  /* Get a line from the user. */
  line_read = readline ("minivm > ");

  /* If the line has any text in it,
     save it on the history. */
  if (line_read && *line_read)
    add_history (line_read);

  return (line_read);
}

/* Read a command from the user */
void
prompt_user()
{
  while (1)
    {
      rl_gets();
      int len = strlen(line_read);
      if (strncmp(line_read, "step", len) == 0)
        break;
      else if (strncmp(line_read, "print", len) == 0)
        {
          print_regs();
        }
      else if (strncmp(line_read, "help", len) == 0)
        print_help();
      else if (strncmp(line_read, "dis", len) == 0)
        disassemble(R[31], 20);
      else if (strncmp(line_read, "quit", len) == 0)
        exit_vm();
      else
        print_help();
    }
}

void
print_regs()
{
  int i = 0;
  for (i = 0; i < REGISTERS / 4; ++i)
    {
      printf("  R%.2d = 0x%.8x  R%.2d = 0x%.8x  R%.2d = 0x%.8x  R%.2d = 0x%.8x\n", 
        4*i, R[4*i], 4*i+1, R[4*i+1],
        4*i+2, R[4*i+2], 4*i+3, R[4*i+3]);
    }
  printf("\n");
}

void
print_help()
{
  printf("  print -- print the contents of registers\n");
  printf("  step  -- execute one instruction\n");
  printf("  dis   -- disassemble several instructions at current PC\n");
  printf("  quit  -- quit the vm\n");
  printf("  help  -- print this help screen\n");
}
