#ifndef VM_H_
#define VM_H_

#include "../config.h"
#include "../loader/loader.h"
#include "../memory/memory.h"

/*
 * Initialize the virtual machine: allocate memory and
 * call the loader module.
 */
void
init_vm();

/* Release memory and exit the VM. */
extern void
exit_vm();

/* The garbage collector */
extern garbage_collector_t* gc;

/* Return true if the given pointer is inside the code segment. */
extern int
is_code_pointer(int addr);

/* Return true if the given pointer is inside the heap segment. */
extern int
is_heap_pointer(int addr);

/* Make sure the given pointer is not outside the heap segment.
 * Exit the VM is so. */
extern void
check_heap_pointer(int addr);

/* Make sure the given pointer is not outside the code segment.
 * Exit the VM is so. */
extern void
check_code_pointer(int addr);
/* Print a disassemble of code_mem to stdout. */
void
disassemble(int start, int size);

/* Block of memory which contains the decoded program. */
extern char* code_mem;

/* Block of memory used for heap allocation. */
extern char* heap_mem;

/* The registers. */
extern int R[REGISTERS];

/* The address of the last instruction of the program. */
extern char* end_program;

/* The size of one word. */
#define WORD_SIZE 4

/* Virtual machine instructions. */

#define OP_LINT  1
#define OP_ADD   2
#define OP_SUB   3
#define OP_MUL   4
#define OP_DIV   5
#define OP_MOD   6

#define OP_ISLT  7
#define OP_ISLE  8
#define OP_ISEQ  9
#define OP_CMOV 10

#define OP_ALOC 11
#define OP_LOAD 12
#define OP_STOR 13

#define OP_RINT 14
#define OP_PINT 15
#define OP_RCHR 16
#define OP_PCHR 17
#define OP_HALT 18

/* The number of opcodes. */
#define NOPCODES 18

#define PUT_WORD(pc, val)  ((*(int*)(code_mem + pc)) = (val))
#define GET_WORD(pc) (*(int*)(pc))

#define PC ((char*)(R[31]))

/* This macro relies on the fact that tokens have consecutive numbers
 * assigned in tokens.h and they have the same order as here! */
#define TOK2OP(tok) (tok - 9)

#endif /*VM_H_*/
