/*
 * MiniVM project (c) LAMP 2006
 * 
 * Functions that deal with resolving labels in the input
 * file.
 * 
 * $Id: labels.c,v 1.1 2006/03/16 15:55:26 dragos Exp $
 */

#define MAX_LABELS 10000

#include <stdlib.h>
#include <string.h>
#include "../kazlib/hash.h"
#include "../main.h"
#include "../emulator/vm.h"
#include "labels.h"


static hash_t* labels = NULL;

static void
free_hnode(hnode_t* node, void* ctx);

static hnode_t *
alloc_hnode(void* ctx);


/* Allocate memory for the label hash map. */
void
init_labels()
{
  if (labels == NULL)
    {
      labels = hash_create(MAX_LABELS, NULL, NULL);
      if (labels == NULL)
        {
          printf("Could not create hash table for labels of %d elements.\n",
                 MAX_LABELS);
          exit_vm();
        }
      /* Set our de-allocator, which knows about labels */
      hash_set_allocator(labels, alloc_hnode, free_hnode, NULL);
    }
}

/* Add a label, and return it. If the label already exists,
 * return the existing one. */
label_t*
add_label(const char* ident)
{
  if (labels == NULL)
    init_labels();
        
  hnode_t* node = hash_lookup(labels, ident);
  if (node == NULL)
    {
      label_t* l = create_label();            
      hash_insert(labels, hnode_create(l), ident);
      return l;
    } 
  else
    {
      return (label_t*)hnode_get(node);
    }
}       

/* Return the label associated to this identifier, or NULL
 * if it's not found. */
label_t*
get_label(const char* ident)
{
  if (labels == NULL)
    init_labels();
        
  hnode_t* node = hash_lookup(labels, ident);
  if (node == NULL)
    return NULL;
  else
    return (label_t*)hnode_get(node);
}

/* Create and initialize a label. */
label_t*
create_label()
{
  label_t* label = malloc(sizeof(label_t));
  if (label == NULL)
    {
      printf("Could not allocate memory for label.\n");
      exit_vm();
    }
  label->address = 0;
  label->anchored = FALSE;
  label->nuses = 0;
                
  return label;
}

/* called from the hash table implementation, to deallocate nodes. */
void
free_hnode(hnode_t* node, void* ctx)
{
  free(hnode_get(node));
}

static hnode_t *
alloc_hnode(void *context)
{
  return malloc(sizeof (hnode_t));
}


/* Free all memory allocated for the labels */
void
free_labels()
{
  if (labels != NULL)
    {
      hash_free(labels);
    }
}

/* Replace all references to 'label' by the hard address passed in
 * 'addr'. */
void
patch_code(label_t* label, int addr)
{
  int i = 0;
  for (i = 0; i < label->nuses; ++i)
    PUT_WORD(label->uses[i], (int)(code_mem + addr));
}

