/*
 * MiniVM project (c) LAMP 2006
 * 
 * Main function.
 * 
 * $Id: main.c,v 1.2 2006/03/23 16:36:47 dragos Exp $
 */
 
#include <getopt.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "config.h"
#include "main.h"
#include "log.h"
#include "emulator/vm.h"
#include "loader/alignloader.h"

int code_size   = DEFAULT_CODE_SIZE;
int heap_size   = DEFAULT_HEAP_SIZE;

int interactive = FALSE;
int threaded_mode = FALSE;

char* filename  = "<stdin>";
FILE* inputfile = NULL;

/* The options that our VM understands */
static struct option vm_options[] = 
  {
    {"code", required_argument, NULL, 'c'},
    {"heap", required_argument, NULL, 'e'},
    {"help", no_argument, NULL, 'h'},
    {"version", no_argument, NULL, 'v'},
    {"interactive", no_argument, NULL, 'i'},
    {"threaded", no_argument, NULL, 't'},
    {"log", required_argument, NULL, 'g'},
    {"gc", required_argument, NULL, 'm'},
    {0, 0, 0, 0}
  };

void usage();

int
main(int argc, char* argv[])
{
  while (1)
    {
      int c = getopt_long(argc, argv, "c:e:hvitg:", vm_options, NULL);
                
      /* the end of option parsing */
      if (c == -1)
        break;
                        
      switch (c)
        {
        case 'c':
          code_size = atoi(optarg);
          if (code_size == 0)
            {
              printf("Invalid code size: %s\n", optarg);
              return 1;
            }
          break;
                                
        case 'e':
          heap_size = atoi(optarg);
          if (heap_size == 0)
            {
              printf("Invalid heap size: %s\n", optarg);
              return 1;
            }
          break;
                        
        case 'h':
          usage();
          return 0;
                                
        case 'v':
          printf("%s - %s\n", PROGRAM_NAME, VERSION);
          return 0;
          
        case 'i':
          interactive = TRUE;
          break;

        case 't':
          printf("Threaded mode not implemented.");
          threaded_mode = TRUE;
          break;

        case 'g':
          log_level = atoi(optarg);
          if (log_level == 0)
            {
              printf("Invalid log level: %s\n", optarg);
              return 1;
            }
          break;

        case 'm':
          if (strncmp(optarg, "none", strlen(optarg)) == 0)
            gc = &no_gc;
          else 
          {
            printf("Invalid garbage collector: %s\n", optarg);
            return 1;
          }
          break;


        case '?':
          usage();
          return 0;
        }
    } /* end of option parsing */
        
  /* open input file, if none given use stdin */
  if (optind < argc)
    {
      filename = argv[optind];
      inputfile = fopen(filename, "r");
      if (inputfile == NULL)
        {
          perror(filename);
          return 1;
        }
    } else
      inputfile = stdin;
        
  if (!init_log(LOG_FILE))
    return 1;

  write_log(LOG_INFO, "%s started with heap: %4dk and code: %4dk\n", 
            PROGRAM_NAME, heap_size / 1024, code_size / 1024);
        
  init_vm();
        
  return 0;
}


/* Print usage information. */
void
usage()
{
  printf("%s\n", PROGRAM_NAME);
  printf("Usage: minivm [options] input_file\n\n");
  printf("  -c,\t--%s\t\tSet code memory size (%d)\n", vm_options[0].name, DEFAULT_CODE_SIZE);
  printf("  -e,\t--%s\t\tSet heap memory size (%d)\n", vm_options[1].name, DEFAULT_HEAP_SIZE);
  printf("  -h,\t--%s\t\tShow this help screen\n",     vm_options[2].name);
  printf("  -v,\t--%s\tPrint version information\n",   vm_options[3].name);
  printf("  -i,\t--%s\tInteractive execution mode\n",  vm_options[4].name);
  printf("  -t,\t--%s\tDirect threaded execution\n",   vm_options[5].name);  
  printf("  -g,\t--%s\t\tLog level (0 - ERROR .. 3 - DEBUG)\n",   vm_options[6].name);  
  printf("   \t--%s\t\tGarbage collector (marksweep, none)\n",   vm_options[7].name);  
}
